<?php

/**
 * Chatwee is a feature-rich social chat platform that provides real-time chatting experience for communities on forums
 *
 * @package Chatwee Plugin for MyBB
 * @version 2.2 Release
 * @date 03.03.2017
 * @author Wojciech Majerski <wojciech.majerski@chatwee.com>
 * @license http://opensource.org/licenses/mit-license.php MIT license
 */

if(!defined("IN_MYBB"))
{
	die("Direct initialization of this file is not allowed.<br /><br />Please make sure IN_MYBB is defined.");
}

$plugins->add_hook("pre_output_page","pre_output_page_callback");
$plugins->add_hook("member_do_login_end","member_do_login_end_callback");
$plugins->add_hook("member_logout_end","member_logout_end_callback");


/**
 *
 *	Chatwee V2 SDK:
 *
 **/

class ChatweeV2_Configuration
{
	private static $_chatId = null;

	private static $_clientKey = null;

	private static $_customUserAgent = null;

	public static function setChatId($chatId) {
		self::$_chatId = $chatId;
	}

	public static function setClientKey($clientKey) {
		self::$_clientKey = $clientKey;
	}

	public static function setCustomUserAgent($customUserAgent) {
		self::$_customUserAgent = $customUserAgent;
	}

	public static function getChatId() {
		return self::$_chatId;
	}

	public static function getClientKey() {
		return self::$_clientKey;
	}

	public static function getCustomUserAgent() {
		return self::$_customUserAgent;
	}

	public static function isConfigurationSet() {
		return self::$_chatId !== null && self::$_clientKey !== null;
	}
}


class ChatweeV2_HttpClient
{
	const API_URL = "http://chatwee-api.com/v2/";

	const USER_AGENT_HEADER = "ChatweeV2 PHP SDK 1.02 RAW";

	private $response;

	private $responseStatus;

	private $responseObject;

    public function get($method, $parameters) {
		if(ChatweeV2_Configuration::isConfigurationSet() === false) {
			throw new Exception("The client credentials are not set");
		}

		$parameters["chatId"] = ChatweeV2_Configuration::getChatId();
		$parameters["clientKey"] = ChatweeV2_Configuration::getClientKey();

		$serializedParameters = self::_serializeParameters($parameters);
		$url = self::API_URL . $method . "?" . $serializedParameters;

		self::call("GET", $url);
    }

	private function call($method, $url) {
		$curl = curl_init();

		$customUserAgent = ChatweeV2_Configuration::getCustomUserAgent();
		$userAgent = $customUserAgent ? $customUserAgent : self::USER_AGENT_HEADER;

		curl_setopt($curl, CURLOPT_TIMEOUT, 60);
		curl_setopt($curl, CURLOPT_CUSTOMREQUEST, $method);
		curl_setopt($curl, CURLOPT_URL, $url);
		curl_setopt($curl, CURLOPT_ENCODING, "gzip");
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($curl, CURLOPT_HTTPHEADER, Array (
			"Accept: application/xml",
			"Content-Type: application/xml",
			"User-Agent: " . $userAgent
		));

		$this->response = curl_exec($curl);
		$this->responseStatus = curl_getinfo($curl, CURLINFO_HTTP_CODE);
		curl_close($curl);

		$this->responseObject = $this->response ? json_decode($this->response) : null;
		if($this->responseStatus != 200) {
			$responseError = $this->responseObject ? $this->responseObject->errorMessage : "ChatweeV2 PHP SDK unknown error: " . $this->responseStatus;
			$responseError .= " (" . $url . ")";
			throw new Exception($responseError);
		}
    }

	private function _serializeParameters($parameters) {
		if(!is_array($parameters) || count($parameters) == 0) {
			return "";
		}

		$result = "";
		foreach($parameters as $key => $value) {
			$result .= ($key . "=" . ($value ? urlencode($value) : $value) . "&");
		}

		$result = substr_replace($result, "", -1);
		return $result;
	}

	public function getResponse() {
		return $this->response;
	}

	public function getResponseObject() {
		return $this->responseObject;
	}

	public function getResponseStatus() {
		return $this->responseStatus;
	}
}

class ChatweeV2_Session
{
	const SESSION_DURATION = 86400;

	private static function getCookieKey() {
		if(ChatweeV2_Configuration::isConfigurationSet() === false) {
			throw new Exception("The client credentials are not set");
		}
		return "chatwee-SID-" . ChatweeV2_Configuration::getChatId();
	}

    public static function getSessionId() {
    	$cookieKey = self::getCookieKey();

    	return isSet($_COOKIE[$cookieKey]) ? $_COOKIE[$cookieKey] : null;
    }

    public static function setSessionId($sessionId) {
		$hostChunks = explode(".", $_SERVER["HTTP_HOST"]);

		$hostChunks = array_slice($hostChunks, -2);

		$cookieDomain = "." . implode(".", $hostChunks);

		setcookie(self::getCookieKey(), $sessionId, time() + self::SESSION_DURATION, "/", $cookieDomain);
    }

    public static function clearSessionId() {
		$hostChunks = explode(".", $_SERVER["HTTP_HOST"]);

		$hostChunks = array_slice($hostChunks, -2);

		$domain = "." . implode(".", $hostChunks);

		setcookie(self::getCookieKey(), "", time() - 1, "/", $domain);
    }

    public static function isSessionSet() {
		return ChatweeV2_Session::getSessionId() !== null;
    }
}

class ChatweeV2_SsoManager {

	public static function registerUser($parameters) {
    	if(isSet($parameters["login"]) === false) {
    		throw new Exception("login parameter is required");
    	}

		$userId = ChatweeV2_SsoUser::register(Array(
			"login" => $parameters["login"],
			"isAdmin" => isSet($parameters["isAdmin"]) === true ? $parameters["isAdmin"] : false,
			"avatar" => isSet($parameters["avatar"]) === true ? $parameters["avatar"] : ""
		));

		return $userId;
	}

    public static function loginUser($parameters) {
		if(isSet($parameters["userId"]) === false) {
			throw new Exception("userId parameter is required");
		}
    	if(self::isLogged() === true) {
    		self::logoutUser();
    	}

		$sessionId = ChatweeV2_SsoUser::login(Array(
			"userId" => $parameters["userId"],
			"userIp" => ChatweeV2_Utils::getUserIp()
		));

		ChatweeV2_Session::setSessionId($sessionId);

		return $sessionId;
    }

	public static function logoutUser() {
		if(self::isLogged() === false) {
			return false;
		}
		$sessionId = ChatweeV2_Session::getSessionId();

		ChatweeV2_SsoUser::removeSession(Array(
			"sessionId" => $sessionId
		));

		ChatweeV2_Session::clearSessionId();
	}

    public static function editUser($parameters) {
    	if(isSet($parameters["login"]) === false) {
    		throw new Exception("login parameter is required");
    	}
		if(isSet($parameters["userId"]) === false) {
			throw new Exception("userId parameter is required");
		}

		$editParameters = Array(
			"userId" => $parameters["userId"],
			"login" => $parameters["login"],
			"avatar" => isSet($parameters["avatar"]) === true ? $parameters["avatar"] : ""
		);

		if(isSet($parameters["isAdmin"]) === true) {
			$editParameters["isAdmin"] = $parameters["isAdmin"];
		}

		ChatweeV2_SsoUser::edit($editParameters);
    }

    private static function isLogged() {
    	return ChatweeV2_Session::isSessionSet();
    }
}

class ChatweeV2_SsoUser {

	public static function register($parameters) {
		$requestParameters = Array(
			"login" => $parameters["login"],
			"isAdmin" => $parameters["isAdmin"] === true ? 1 : 0,
			"avatar" => $parameters["avatar"]
		);

		$httpClient = new ChatweeV2_HttpClient();
		$httpClient->get("sso-user/register", $requestParameters);

		$userId = $httpClient->getResponseObject();

		return $userId;
	}

    public static function login($parameters) {
		$requestParameters = Array(
			"userId" => $parameters["userId"]
		);

		if(isSet($parameters["userIp"])) {
			if(filter_var($parameters["userIp"], FILTER_VALIDATE_IP, FILTER_FLAG_IPV4) !== false) {
				$requestParameters["userIp"] = $parameters["userIp"];
			}
		}

		$httpClient = new ChatweeV2_HttpClient();
		$httpClient->get("sso-user/login", $requestParameters);

		$sessionId = $httpClient->getResponseObject();

		return $sessionId;
    }

	public static function removeSession($parameters) {
		$requestParameters = Array(
			"sessionId" => $parameters["sessionId"]
		);

		$httpClient = new ChatweeV2_HttpClient();
		$httpClient->get("sso-user/remove-session", $requestParameters);
	}

	public static function validateSession($parameters) {
		$requestParameters = Array(
			"sessionId" => $parameters["sessionId"]
		);

		$httpClient = new ChatweeV2_HttpClient();
		$httpClient->get("sso-user/validate-session", $requestParameters);

		$validationResponse = $httpClient->getResponseObject();

		return $validationResponse;
	}

	public static function logout($parameters) {
		$requestParameters = Array(
			"userId" => $parameters["userId"]
		);

		$httpClient = new ChatweeV2_HttpClient();
		$httpClient->get("sso-user/logout", $requestParameters);
	}

    public static function edit($parameters) {
		$requestParameters = Array(
			"userId" => $parameters["userId"],
			"login" => $parameters["login"],
			"avatar" => $parameters["avatar"]
		);

		if(isSet($parameters["isAdmin"]) === true) {
			$requestParameters["isAdmin"] = $parameters["isAdmin"] === true ? 1 : 0;
		}

		$httpClient = new ChatweeV2_HttpClient();
		$httpClient->get("sso-user/edit", $requestParameters);
    }
}

class ChatweeV2_Utils {

	public static function getUserIp() {
		if (!empty($_SERVER["HTTP_CLIENT_IP"])) {
			$ip = $_SERVER["HTTP_CLIENT_IP"];
		} elseif (!empty($_SERVER["HTTP_X_FORWARDED_FOR"])) {
			$ip = $_SERVER["HTTP_X_FORWARDED_FOR"];
		} else {
			$ip = $_SERVER["REMOTE_ADDR"];
		}
		return $ip;
	}
}

function chatwee_install() {
	global $db;

	$collation = $db->build_create_table_collation();

	if(!$db->table_exists("chatwee_users"))
	{
		switch($db->type)
		{
			case "pgsql":
				$db->write_query("CREATE TABLE ".TABLE_PREFIX."chatwee_users (
					user_id smallint,
					chatwee_id varchar(80),
					PRIMARY KEY (user_id)
				);");
				break;
			case "sqlite":
				$db->write_query("CREATE TABLE ".TABLE_PREFIX."chatwee_users (
					user_id INTEGER PRIMARY KEY,
					chatwee_id varchar(80)
				);");
				break;
			default:
				$db->write_query("CREATE TABLE ".TABLE_PREFIX."chatwee_users (
					user_id int,
					chatwee_id varchar(80),
					PRIMARY KEY (user_id)
				) ENGINE=MyISAM{$collation};");
				break;
		}
	}
}

function chatwee_is_installed() {
	global $db;

	return $db->table_exists('chatwee_users');
}

function chatwee_uninstall() {
	global $db, $mybb;

	if($mybb->request_method != "post") {

		global $page;

		$page->output_confirm_action("index.php?module=config-plugins&action=deactivate&uninstall=1&plugin=chatwee", "Do you wish to delete all Chatwee data from the database? You may want to leave it intact, in case you want to easily retry the plugin in the future.", "MyBB Chat Plugin by Chatwee uninstallation");
	}
}

function chatwee_info() {
	return array(
		"name"			=> "MyBB Chat Plugin by Chatwee",
		"description"	=> "Chatwee chat plugin lets your site visitors interact with each other in real time via media-rich, private, group and public conversations. Customize it to your liking, assign moderators, put various privacy settings in place, and more.",
		"website"		=> "https://chatwee.com/",
		"author"		=> "Chatwee Team",
		"authorsite"	=> "https://chatwee.com/",
		"version"		=> "2.2",
		"compatibility" => "14*,16*,17*,18*",
		"guid" 			=> "",
		"codename"      => "chatwee"
	);
}

function chatwee_activate() {
	global $db;

	$insertarray = array(
		"name" => "chatwee",
		"title" => "MyBB Chat Plugin by Chatwee settings",
		"description" => "Click here to edit the chat settings.",
		"disporder" => 35,
		"isdefault" => 0,
	);
	$gid = $db->insert_query("settinggroups", $insertarray);

	$insertarray = array(
		"name" => "chatwee_code",
		"title" => "Chatwee installation code",
		"description" => mres("Copy the Chatwee installation code from your <a href='https://client.chatwee.com/v2/dashboard' target='_blank'>Dashboard</a> and paste it into the box. If you don't have a Chatwee account yet, please <a href='https://client.chatwee.com/register-form/v2' target='_blank'>sign up</a> absolutely for free."),
		"optionscode" => "textarea",
		"value" => "",
		"disporder" => 2,
		"gid" => $gid
	);
	$db->insert_query("settings", $insertarray);

	$insertarray = array(
		"name" => "chatwee_sso_enable",
		"title" => "Enable SSO",
		"description" => "Check this box if you want your users to log in via Single Sign-on.",
		"optionscode" => "checkbox \n1= ",
		"value" => 0,
		"disporder" => 3,
		"isdefault" => 0,
		"gid" => $gid
	);
	$db->insert_query("settings", $insertarray);

	$insertarray = array(
		"name" => "chatwee_chatid",
		"title" => "Chat ID",
		"description" => mres("Enter your Chat ID available in the <a href='https://client.chatwee.com/v2/customize#integration' target='_blank'>Integration</a> tab."),
		"optionscode" => "text",
		"value" => "",
		"disporder" => 5,
		"gid" => $gid
	);
	$db->insert_query("settings", $insertarray);

	$insertarray = array(
		"name" => "chatwee_apikey",
		"title" => "API Key",
		"description" => mres("Enter your Client API Key available in the <a href='https://client.chatwee.com/v2/customize#integration' target='_blank'>Integration</a> tab."),
		"optionscode" => "text",
		"value" => "",
		"disporder" => 6,
		"gid" => $gid
	);
	$db->insert_query("settings", $insertarray);

	$insertarray = array(
		"name" => "chatwee_global_moderators_as_admins",
		"title" => "Super Moderators as Chatwee Admins",
		"description" => "Check this box if you want to assign admin powers to Super Moderators.",
		"optionscode" => "checkbox \n1= ",
		"value" => 0,
		"disporder" => 7,
		"isdefault" => 0,
		"gid" => $gid
	);
	$db->insert_query("settings", $insertarray);

	rebuild_settings();
}

function chatwee_deactivate() {
	global $db;

	$db->delete_query("settings", "name IN('chatwee_code', 'chatwee_sso_enable', 'chatwee_chatid', 'chatwee_apikey', 'chatwee_global_moderators_as_admins')");
	$db->delete_query("settinggroups", "name IN('chatwee')");
}

function mres($value) {
    $search = array("\\",  "\x00", "\n",  "\r",  "'",  '"', "\x1a");
    $replace = array("\\\\","\\0","\\n", "\\r", "\'", '\"', "\\Z");

    return str_replace($search, $replace, $value);
}

function log_error($message) {
	//TBI: no log engine supported
}

function pre_output_page_callback($page) {
	global $mybb;

	if($mybb->settings["chatwee_code"]) {
		$page = str_replace("</body>", $mybb->settings["chatwee_code"] . "</body>", $page);
	}

	synchronize_sso_user();

	return $page;
}

function member_do_login_end_callback() {
	global $mybb;

    $sso_enable = $mybb->settings["chatwee_sso_enable"];

    if($sso_enable == 1) {

       	if($mybb->input) {
			$user = validate_password_from_username($mybb->input["username"], $mybb->input["password"]);
		}
		if(!$user["uid"]) {
			return;
		}
		$user = get_user($user["uid"]);

		sso_login($user);
    }
}

function member_logout_end_callback() {
	global $mybb;

    $sso_enable = $mybb->settings["chatwee_sso_enable"];

    if($sso_enable == 1) {
		sso_logout();
    }
}

function set_sso_credentials() {
    global $mybb;

    $chatId = $mybb->settings["chatwee_chatid"];
    $clientKey = $mybb->settings["chatwee_apikey"];

	ChatweeV2_Configuration::setChatId($chatId);
	ChatweeV2_Configuration::setClientKey($clientKey);
	ChatweeV2_Configuration::setCustomUserAgent("Chatwee myBB Plugin 2.2");
}

function synchronize_sso_user() {
	global $mybb;

	log_error("Error while reinitializng SSO session");

	if(!empty($_SERVER["HTTP_X_REQUESTED_WITH"]) && strtolower($_SERVER["HTTP_X_REQUESTED_WITH"]) === "xmlhttprequest") {
		return false;
	}

	if($mybb->input['action'] == "logout") {
		return false;
	}

	$sso_enable = $mybb->settings["chatwee_sso_enable"];
	$user = $mybb->user;

	if($sso_enable == 1 && $user['uid']) {

		set_sso_credentials();

		try {
			if(ChatweeV2_Session::isSessionSet() === false) {
				sso_login($user);
			} else {
				$sessionId = ChatweeV2_Session::getSessionId();

				$isSessionValid = ChatweeV2_SsoUser::validateSession(Array(
					"sessionId" => $sessionId
				));

				if($isSessionValid === false) {
					sso_login($user);
				}
			}
        } catch(Exception $exception) {
            log_error("Error while reinitializng SSO session: " . $exception->getMessage());
        }
	}
}

function sso_login($user) {

    global $mybb;

    if($mybb->settings["chatwee_chatid"] && $mybb->settings["chatwee_apikey"]) {

    	set_sso_credentials();

	    $user_chatwee_id = fetch_user_chatwee_id($user);

	    if (!$user_chatwee_id) {
	        $user_chatwee_id = sso_register($user);
	    }

		try	{
			ChatweeV2_SsoUser::edit(Array(
				"userId" => $user_chatwee_id,
				"login" => $data->user_update_data["username"] ? $data->user_update_data["username"] : $user["username"],
				"avatar" => $user["avatar"],
				"isAdmin" => is_chatwee_admin($user)
			));
		} catch(Exception $exception) {
			if(strpos($exception->getMessage(), "User does not exist") !== false) {
				$user_chatwee_id = sso_register($user);
			} else {
				log_error("Error while executing sso_login: " . $exception->getMessage());
			}
		}

		try {
			ChatweeV2_SsoManager::loginUser(Array(
				"userId" => $user_chatwee_id
			));
		} catch(Exception $exception) {
			log_error("Error while executing sso_login: " . $exception->getMessage());
		}
	}

}

function fetch_user_chatwee_id($user) {
	global $db;

	$user_id = $user["uid"];

	$query = $db->simple_select("chatwee_users", "chatwee_id", "user_id = " . $user_id, array(
	    "limit" => 1
	));

	$result = $db->fetch_array($query);

	return $result ? $result["chatwee_id"] : null;
}

function sso_register($user) {

    try {
        $user_chatwee_id = ChatweeV2_SsoManager::registerUser(Array(
            "login" => $user["username"],
            "avatar" => $user["avatar"],
            "isAdmin" => is_chatwee_admin($user)
        ));

		global $db;
	    $user_id = $user["uid"];

	    $db->delete_query("chatwee_users", "user_id='$user_id'");
	    $db->insert_query("chatwee_users", array("user_id" => $user_id, "chatwee_id" => $user_chatwee_id));

	    return $user_chatwee_id;
    } catch(Exception $exception) {
        log_error("Error while executing sso_register: " . $exception->getMessage());
    }
    return $userId;
}


function sso_logout() {

    set_sso_credentials();

    try {
        ChatweeV2_SsoManager::logoutUser();
    } catch(Exception $exception) {
       log_error("Error while executing sso_logout: " . $exception->getMessage());
    }
}

function is_chatwee_admin($user = null) {
    global $mybb;

    $user_additional_groups = explode(",", $user["additionalgroups"]);

    if($user["usergroup"] == 4 || in_array(4, $user_additional_groups)) {
		return true;
    } elseif ($mybb->settings["chatwee_global_moderators_as_admins"] == 1 && ($user["usergroup"] == 3 || in_array(3, $user_additional_groups))) {
        return true;
    }

    if ($mybb->settings["chatwee_global_moderators_as_admins"] == 1) {
	    if (check_if_is_supermod($user["usergroup"]) === true) {
	    	return true;
	    }

	    if (!empty($user["additionalgroups"])) {
		    foreach ($user_additional_groups as $group) {
		    	if (check_if_is_supermod($group) === true) {
		    		return true;
		    	}
		    }
		}
	}

    return false;
}

function check_if_set_as_chat_admin($user) {
	global $db;

	$user_id = $user["uid"];

	$query = $db->simple_select("chatwee_users", "set_as_chat_admin", "user_id = " . $user_id, array(
	    "limit" => 1
	));

	$row = $db->fetch_array($query);
	$row["set_as_chat_admin"] == 1 ? $result = true : $result = false;

	return $result;
}

function check_if_is_supermod($gid) {
	global $db;

	$query = $db->simple_select("usergroups", "issupermod", "gid = " . $gid);

	$row = $db->fetch_array($query);
	$row["issupermod"] == 1 ? $result = true : $result = false;

	return $result;
}
